# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import base64
from typing import Dict, MutableMapping, Optional
from urllib import parse


class HttpChallenge(object):
    """An object representing the content of a Key Vault authentication challenge.

    :param str request_uri: The URI of the HTTP request that prompted this challenge.
    :param str challenge: The WWW-Authenticate header of the challenge response.
    :param response_headers: Optional. The headers attached to the challenge response.
    :type response_headers: MutableMapping[str, str] or None
    """

    def __init__(
        self, request_uri: str, challenge: str, response_headers: "Optional[MutableMapping[str, str]]" = None
    ) -> None:
        """Parses an HTTP WWW-Authentication Bearer challenge from a server.

        Example challenge with claims:
            Bearer authorization="https://login.windows-ppe.net/", error="invalid_token",
            error_description="User session has been revoked",
            claims="eyJhY2Nlc3NfdG9rZW4iOnsibmJmIjp7ImVzc2VudGlhbCI6dHJ1ZSwgInZhbHVlIjoiMTYwMzc0MjgwMCJ9fX0="
        """
        self.source_authority = self._validate_request_uri(request_uri)
        self.source_uri = request_uri
        self._parameters: "Dict[str, str]" = {}

        # get the scheme of the challenge and remove from the challenge string
        trimmed_challenge = self._validate_challenge(challenge)
        split_challenge = trimmed_challenge.split(" ", 1)
        self.scheme = split_challenge[0]
        trimmed_challenge = split_challenge[1]

        self.claims = None
        # split trimmed challenge into comma-separated name=value pairs. Values are expected
        # to be surrounded by quotes which are stripped here.
        for item in trimmed_challenge.split(","):
            # Special case for claims, which can contain = symbols as padding. Assume at most one claim per challenge
            if "claims=" in item:
                encoded_claims = item[item.index("=") + 1 :].strip(" \"'")
                padding_needed = -len(encoded_claims) % 4
                try:
                    decoded_claims = base64.urlsafe_b64decode(encoded_claims + "=" * padding_needed).decode()
                    self.claims = decoded_claims
                except Exception:  # pylint:disable=broad-except
                    continue
            # process name=value pairs
            else:
                comps = item.split("=")
                if len(comps) == 2:
                    key = comps[0].strip(' "')
                    value = comps[1].strip(' "')
                    if key:
                        self._parameters[key] = value

        # minimum set of parameters
        if not self._parameters:
            raise ValueError("Invalid challenge parameters")

        # must specify authorization or authorization_uri
        if "authorization" not in self._parameters and "authorization_uri" not in self._parameters:
            raise ValueError("Invalid challenge parameters")

        authorization_uri = self.get_authorization_server()
        # the authorization server URI should look something like https://login.windows.net/tenant-id
        raw_uri_path = str(parse.urlparse(authorization_uri).path)
        uri_path = raw_uri_path.lstrip("/")
        self.tenant_id = uri_path.split("/", maxsplit=1)[0] or None

        # if the response headers were supplied
        if response_headers:
            # get the message signing key and message key encryption key from the headers
            self.server_signature_key = response_headers.get("x-ms-message-signing-key", None)
            self.server_encryption_key = response_headers.get("x-ms-message-encryption-key", None)

    def is_bearer_challenge(self) -> bool:
        """Tests whether the HttpChallenge is a Bearer challenge.

        :returns: True if the challenge is a Bearer challenge; False otherwise.
        :rtype: bool
        """
        if not self.scheme:
            return False

        return self.scheme.lower() == "bearer"

    def is_pop_challenge(self) -> bool:
        """Tests whether the HttpChallenge is a proof of possession challenge.

        :returns: True if the challenge is a proof of possession challenge; False otherwise.
        :rtype: bool
        """
        if not self.scheme:
            return False

        return self.scheme.lower() == "pop"

    def get_value(self, key: str) -> "Optional[str]":
        return self._parameters.get(key)

    def get_authorization_server(self) -> str:
        """Returns the URI for the authorization server if present, otherwise an empty string.

        :returns: The URI for the authorization server if present, otherwise an empty string.
        :rtype: str
        """
        value = ""
        for key in ["authorization_uri", "authorization"]:
            value = self.get_value(key) or ""
            if value:
                break
        return value

    def get_resource(self) -> str:
        """Returns the resource if present, otherwise an empty string.

        :returns: The challenge resource if present, otherwise an empty string.
        :rtype: str
        """
        return self.get_value("resource") or ""

    def get_scope(self) -> str:
        """Returns the scope if present, otherwise an empty string.

        :returns: The challenge scope if present, otherwise an empty string.
        :rtype: str
        """
        return self.get_value("scope") or ""

    def supports_pop(self) -> bool:
        """Returns True if the challenge supports proof of possession token auth; False otherwise.

        :returns: True if the challenge supports proof of possession token auth; False otherwise.
        :rtype: bool
        """
        return self._parameters.get("supportspop", "").lower() == "true"

    def supports_message_protection(self) -> bool:
        """Returns True if the challenge vault supports message protection; False otherwise.

        :returns: True if the challenge vault supports message protection; False otherwise.
        :rtype: bool
        """
        return self.supports_pop() and self.server_encryption_key and self.server_signature_key  # type: ignore

    def _validate_challenge(
        self, challenge: str
    ) -> str:  # pylint:disable=bad-option-value,useless-option-value,no-self-use
        """Verifies that the challenge is a valid auth challenge and returns the key=value pairs.

        :param str challenge: The WWW-Authenticate header of the challenge response.

        :returns: The challenge key/value pairs, with whitespace removed, as a string.
        :rtype: str
        """
        if not challenge:
            raise ValueError("Challenge cannot be empty")

        return challenge.strip()

    def _validate_request_uri(
        self, uri: str
    ) -> str:  # pylint:disable=bad-option-value,useless-option-value,no-self-use
        """Extracts the host authority from the given URI.

        :param str uri: The URI of the HTTP request that prompted the challenge.

        :returns: The challenge host authority.
        :rtype: str
        """
        if not uri:
            raise ValueError("request_uri cannot be empty")

        parsed = parse.urlparse(uri)
        if not parsed.netloc:
            raise ValueError("request_uri must be an absolute URI")

        if parsed.scheme.lower() not in ["http", "https"]:
            raise ValueError("request_uri must be HTTP or HTTPS")

        return parsed.netloc



# pylint: disable=too-many-lines,too-many-statements
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
from typing import Any, Callable, Dict, IO, Optional, TypeVar, Union, overload

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from .._serialization import Serializer
from .._vendor import PostgreSQLManagementClientMixinABC, _convert_request

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_check_migration_name_availability_request(  # pylint: disable=name-too-long
    subscription_id: str, resource_group_name: str, target_db_server_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-12-01-preview"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DBforPostgreSQL/flexibleServers/{targetDbServerName}/checkMigrationNameAvailability",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url(
            "subscription_id",
            subscription_id,
            "str",
            pattern=r"([a-z0-9]){8,8}[-]([a-z0-9]){4,4}[-]([a-z0-9]){4,4}[-]([a-z0-9]){4,4}[-]([a-z0-9]){12,12}",
        ),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", pattern=r"^[-a-z0-9A-Z._()]+[^.]$"
        ),
        "targetDbServerName": _SERIALIZER.url(
            "target_db_server_name", target_db_server_name, "str", pattern=r"([-a-z0-9]){3,63}"
        ),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


class PostgreSQLManagementClientOperationsMixin(PostgreSQLManagementClientMixinABC):  # pylint: disable=name-too-long
    @overload
    def check_migration_name_availability(
        self,
        subscription_id: str,
        resource_group_name: str,
        target_db_server_name: str,
        parameters: _models.MigrationNameAvailabilityResource,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MigrationNameAvailabilityResource:
        """Check migration name validity and availability.

        This method checks whether a proposed migration name is valid and available.

        :param subscription_id: The subscription ID of the target database server. Required.
        :type subscription_id: str
        :param resource_group_name: The resource group name of the target database server. Required.
        :type resource_group_name: str
        :param target_db_server_name: The name of the target database server. Required.
        :type target_db_server_name: str
        :param parameters: The required parameters for checking if a migration name is available.
         Required.
        :type parameters:
         ~azure.mgmt.rdbms.postgresql_flexibleservers.models.MigrationNameAvailabilityResource
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MigrationNameAvailabilityResource or the result of cls(response)
        :rtype: ~azure.mgmt.rdbms.postgresql_flexibleservers.models.MigrationNameAvailabilityResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def check_migration_name_availability(
        self,
        subscription_id: str,
        resource_group_name: str,
        target_db_server_name: str,
        parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MigrationNameAvailabilityResource:
        """Check migration name validity and availability.

        This method checks whether a proposed migration name is valid and available.

        :param subscription_id: The subscription ID of the target database server. Required.
        :type subscription_id: str
        :param resource_group_name: The resource group name of the target database server. Required.
        :type resource_group_name: str
        :param target_db_server_name: The name of the target database server. Required.
        :type target_db_server_name: str
        :param parameters: The required parameters for checking if a migration name is available.
         Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MigrationNameAvailabilityResource or the result of cls(response)
        :rtype: ~azure.mgmt.rdbms.postgresql_flexibleservers.models.MigrationNameAvailabilityResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def check_migration_name_availability(
        self,
        subscription_id: str,
        resource_group_name: str,
        target_db_server_name: str,
        parameters: Union[_models.MigrationNameAvailabilityResource, IO[bytes]],
        **kwargs: Any
    ) -> _models.MigrationNameAvailabilityResource:
        """Check migration name validity and availability.

        This method checks whether a proposed migration name is valid and available.

        :param subscription_id: The subscription ID of the target database server. Required.
        :type subscription_id: str
        :param resource_group_name: The resource group name of the target database server. Required.
        :type resource_group_name: str
        :param target_db_server_name: The name of the target database server. Required.
        :type target_db_server_name: str
        :param parameters: The required parameters for checking if a migration name is available. Is
         either a MigrationNameAvailabilityResource type or a IO[bytes] type. Required.
        :type parameters:
         ~azure.mgmt.rdbms.postgresql_flexibleservers.models.MigrationNameAvailabilityResource or
         IO[bytes]
        :return: MigrationNameAvailabilityResource or the result of cls(response)
        :rtype: ~azure.mgmt.rdbms.postgresql_flexibleservers.models.MigrationNameAvailabilityResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.MigrationNameAvailabilityResource] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "MigrationNameAvailabilityResource")

        _request = build_check_migration_name_availability_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            target_db_server_name=target_db_server_name,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request = _convert_request(_request)
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("MigrationNameAvailabilityResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=logging-fstring-interpolation

import sys
import time
import threading
import urllib
import requests
import websocket

from knack.log import get_logger
from azure.cli.core.azclierror import CLIInternalError, ValidationError
from azure.cli.core.commands.client_factory import get_subscription_id

from ._clients import ContainerAppClient
from ._utils import safe_get, is_platform_windows

# pylint: disable=import-error,ungrouped-imports
if is_platform_windows():
    import msvcrt
    from azure.cli.command_modules.container._vt_helper import (enable_vt_mode, _get_conout_mode,
                                                                _set_conout_mode, _get_conin_mode, _set_conin_mode)

logger = get_logger(__name__)

# SSH control byte values for container app proxy
SSH_PROXY_FORWARD = 0
SSH_PROXY_INFO = 1
SSH_PROXY_ERROR = 2

# SSH control byte values for container app cluster
SSH_CLUSTER_STDIN = 0
SSH_CLUSTER_STDOUT = 1
SSH_CLUSTER_STDERR = 2

# forward byte + stdin byte
SSH_INPUT_PREFIX = b"\x00\x00"

# forward byte + terminal resize byte
SSH_TERM_RESIZE_PREFIX = b"\x00\x04"

SSH_DEFAULT_ENCODING = "utf-8"
SSH_BACKUP_ENCODING = "latin_1"

SSH_CTRL_C_MSG = b"\x00\x00\x03"


class WebSocketConnection:
    def __init__(self, cmd, resource_group_name, name, revision, replica, container, startup_command):
        token_response = ContainerAppClient.get_auth_token(cmd, resource_group_name, name)
        self._token = token_response["properties"]["token"]
        self._logstream_endpoint = self._get_logstream_endpoint(cmd, resource_group_name, name,
                                                                revision, replica, container)
        self._url = self._get_url(cmd=cmd, resource_group_name=resource_group_name, name=name, revision=revision,
                                  replica=replica, container=container, startup_command=startup_command)
        self._socket = websocket.WebSocket(enable_multithread=True)
        logger.info("Attempting to connect to %s", self._url)
        self._socket.connect(self._url, header=[f"Authorization: Bearer {self._token}"])

        self.is_connected = True
        self._windows_conout_mode = None
        self._windows_conin_mode = None
        if is_platform_windows():
            self._windows_conout_mode = _get_conout_mode()
            self._windows_conin_mode = _get_conin_mode()

    @classmethod
    def _get_logstream_endpoint(cls, cmd, resource_group_name, name, revision, replica, container):
        containers = ContainerAppClient.get_replica(cmd,
                                                    resource_group_name,
                                                    name, revision, replica)["properties"]["containers"]
        container_info = [c for c in containers if c["name"] == container]
        if not container_info:
            raise ValidationError(f"No such container: {container}")
        return container_info[0]["logStreamEndpoint"]

    def _get_url(self, cmd, resource_group_name, name, revision, replica, container, startup_command):
        sub = get_subscription_id(cmd.cli_ctx)
        base_url = self._logstream_endpoint
        proxy_api_url = base_url[:base_url.index("/subscriptions/")].replace("https://", "")
        encoded_cmd = urllib.parse.quote_plus(startup_command)

        return (f"wss://{proxy_api_url}/subscriptions/{sub}/resourceGroups/{resource_group_name}/containerApps/{name}"
                f"/revisions/{revision}/replicas/{replica}/containers/{container}/exec"
                f"?command={encoded_cmd}")

    def disconnect(self):
        logger.warning("Disconnecting...")
        self.is_connected = False
        self._socket.close()
        if self._windows_conout_mode and self._windows_conin_mode:
            _set_conout_mode(self._windows_conout_mode)
            _set_conin_mode(self._windows_conin_mode)

    def send(self, *args, **kwargs):
        return self._socket.send(*args, **kwargs)

    def recv(self, *args, **kwargs):
        return self._socket.recv(*args, **kwargs)


def _decode_and_output_to_terminal(connection: WebSocketConnection, response, encodings):
    for i, encoding in enumerate(encodings):
        try:
            print(response[2:].decode(encoding), end="", flush=True)
            break
        except UnicodeDecodeError as e:
            if i == len(encodings) - 1:  # ran out of encodings to try
                connection.disconnect()
                logger.info("Proxy Control Byte: %s", response[0])
                logger.info("Cluster Control Byte: %s", response[1])
                logger.info("Hexdump: %s", response[2:].hex())
                raise CLIInternalError("Failed to decode server data") from e
            logger.info("Failed to encode with encoding %s", encoding)


def read_ssh(connection: WebSocketConnection, response_encodings):
    # We just need to do resize once for the whole session
    _resize_terminal(connection)

    # response_encodings is the ordered list of Unicode encodings to try to decode with before raising an exception
    while connection.is_connected:
        response = connection.recv()
        if not response:
            connection.disconnect()
        else:
            logger.info("Received raw response %s", response.hex())
            proxy_status = response[0]
            if proxy_status == SSH_PROXY_INFO:
                print(f"INFO: {response[1:].decode(SSH_DEFAULT_ENCODING)}")
            elif proxy_status == SSH_PROXY_ERROR:
                print(f"ERROR: {response[1:].decode(SSH_DEFAULT_ENCODING)}")
            elif proxy_status == SSH_PROXY_FORWARD:
                control_byte = response[1]
                if control_byte in (SSH_CLUSTER_STDOUT, SSH_CLUSTER_STDERR):
                    _decode_and_output_to_terminal(connection, response, response_encodings)
                else:
                    connection.disconnect()
                    raise CLIInternalError("Unexpected message received")


def _send_stdin(connection: WebSocketConnection, getch_fn):
    while connection.is_connected:
        ch = getch_fn()
        if connection.is_connected:
            connection.send(b"".join([SSH_INPUT_PREFIX, ch]))


def _resize_terminal(connection: WebSocketConnection):
    from shutil import get_terminal_size
    size = get_terminal_size()
    if connection.is_connected:
        connection.send(b"".join([SSH_TERM_RESIZE_PREFIX,
                                  f'{{"Width": {size.columns}, '
                                  f'"Height": {size.lines}}}'.encode(SSH_DEFAULT_ENCODING)]))


def _getch_unix():
    return sys.stdin.read(1).encode(SSH_DEFAULT_ENCODING)


def _getch_windows():
    while not msvcrt.kbhit():
        time.sleep(0.01)
    return msvcrt.getch()


def ping_container_app(app):
    site = safe_get(app, "properties", "configuration", "ingress", "fqdn")
    if site:
        try:
            resp = requests.get(f'https://{site}', timeout=30)
            if not resp.ok:
                logger.info(f"Got bad status pinging app: {resp.status_code}")
        except requests.exceptions.ReadTimeout:
            logger.info("Timed out while pinging app external URL")
    else:
        logger.info("Could not fetch site external URL")


def get_stdin_writer(connection: WebSocketConnection):
    if not is_platform_windows():
        import tty
        tty.setcbreak(sys.stdin.fileno())  # needed to prevent printing arrow key characters
        writer = threading.Thread(target=_send_stdin, args=(connection, _getch_unix))
    else:
        enable_vt_mode()  # needed for interactive commands (ie vim)
        writer = threading.Thread(target=_send_stdin, args=(connection, _getch_windows))

    return writer



import collections
import copy
from .Utils import _write_complex_object

class PageViewPerfData(object):
    """Data contract class for type PageViewPerfData.
    """
    ENVELOPE_TYPE_NAME = 'Microsoft.ApplicationInsights.PageViewPerfData'	
    	
    DATA_TYPE_NAME = 'PageViewPerfData'

    _defaults = collections.OrderedDict([
        ('ver', 2),
        ('ver', 2),
        ('url', None),
        ('perfTotal', None),
        ('name', None),
        ('name', None),
        ('duration', None),
        ('networkConnect', None),
        ('sentRequest', None),
        ('receivedResponse', None),
        ('id', None),
        ('domProcessing', None),
        ('referrerUri', None),
        ('properties', {}),
        ('properties', {}),
        ('measurements', {}),
    ])
    
    def __init__(self):
        """Initializes a new instance of the class.
        """
        self._values = {
            'ver': 2,
            'name': None,
        }
        self._initialize()
        
    @property
    def ver(self):
        """The ver property.
        
        Returns:
            (int). the property value. (defaults to: 2)
        """
        return self._values['ver']
        
    @ver.setter
    def ver(self, value):
        """The ver property.
        
        Args:
            value (int). the property value.
        """
        self._values['ver'] = value

    @property
    def url(self):
        """The url property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'url' in self._values:
            return self._values['url']
        return self._defaults['url']
        
    @url.setter
    def url(self, value):
        """The url property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['url'] and 'url' in self._values:
            del self._values['url']
        else:
            self._values['url'] = value
        
    @property
    def perf_total(self):
        """The perf_total property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'perfTotal' in self._values:
            return self._values['perfTotal']
        return self._defaults['perfTotal']
        
    @perf_total.setter
    def perf_total(self, value):
        """The perf_total property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['perfTotal'] and 'perfTotal' in self._values:
            del self._values['perfTotal']
        else:
            self._values['perfTotal'] = value
        
    @property
    def name(self):
        """The name property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        return self._values['name']
        
    @name.setter
    def name(self, value):
        """The name property.
        
        Args:
            value (string). the property value.
        """
        self._values['name'] = value
        
    @property
    def duration(self):
        """The duration property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'duration' in self._values:
            return self._values['duration']
        return self._defaults['duration']
        
    @duration.setter
    def duration(self, value):
        """The duration property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['duration'] and 'duration' in self._values:
            del self._values['duration']
        else:
            self._values['duration'] = value
        
    @property
    def network_connect(self):
        """The network_connect property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'networkConnect' in self._values:
            return self._values['networkConnect']
        return self._defaults['networkConnect']
        
    @network_connect.setter
    def network_connect(self, value):
        """The network_connect property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['networkConnect'] and 'networkConnect' in self._values:
            del self._values['networkConnect']
        else:
            self._values['networkConnect'] = value
        
    @property
    def sent_request(self):
        """The sent_request property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'sentRequest' in self._values:
            return self._values['sentRequest']
        return self._defaults['sentRequest']
        
    @sent_request.setter
    def sent_request(self, value):
        """The sent_request property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['sentRequest'] and 'sentRequest' in self._values:
            del self._values['sentRequest']
        else:
            self._values['sentRequest'] = value
        
    @property
    def received_response(self):
        """The received_response property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'receivedResponse' in self._values:
            return self._values['receivedResponse']
        return self._defaults['receivedResponse']
        
    @received_response.setter
    def received_response(self, value):
        """The received_response property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['receivedResponse'] and 'receivedResponse' in self._values:
            del self._values['receivedResponse']
        else:
            self._values['receivedResponse'] = value
        
    @property
    def id(self):
        """The id property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'id' in self._values:
            return self._values['id']
        return self._defaults['id']
        
    @id.setter
    def id(self, value):
        """The id property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['id'] and 'id' in self._values:
            del self._values['id']
        else:
            self._values['id'] = value
        
    @property
    def dom_processing(self):
        """The dom_processing property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'domProcessing' in self._values:
            return self._values['domProcessing']
        return self._defaults['domProcessing']
        
    @dom_processing.setter
    def dom_processing(self, value):
        """The dom_processing property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['domProcessing'] and 'domProcessing' in self._values:
            del self._values['domProcessing']
        else:
            self._values['domProcessing'] = value
        
    @property
    def referrer_uri(self):
        """The referrer_uri property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'referrerUri' in self._values:
            return self._values['referrerUri']
        return self._defaults['referrerUri']
        
    @referrer_uri.setter
    def referrer_uri(self, value):
        """The referrer_uri property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['referrerUri'] and 'referrerUri' in self._values:
            del self._values['referrerUri']
        else:
            self._values['referrerUri'] = value
        
    @property
    def properties(self):
        """The properties property.
        
        Returns:
            (hash). the property value. (defaults to: {})
        """
        if 'properties' in self._values:
            return self._values['properties']
        self._values['properties'] = copy.deepcopy(self._defaults['properties'])
        return self._values['properties']
        
    @properties.setter
    def properties(self, value):
        """The properties property.
        
        Args:
            value (hash). the property value.
        """
        if value == self._defaults['properties'] and 'properties' in self._values:
            del self._values['properties']
        else:
            self._values['properties'] = value
        
    @property
    def measurements(self):
        """The measurements property.
        
        Returns:
            (hash). the property value. (defaults to: {})
        """
        if 'measurements' in self._values:
            return self._values['measurements']
        self._values['measurements'] = copy.deepcopy(self._defaults['measurements'])
        return self._values['measurements']
        
    @measurements.setter
    def measurements(self, value):
        """The measurements property.
        
        Args:
            value (hash). the property value.
        """
        if value == self._defaults['measurements'] and 'measurements' in self._values:
            del self._values['measurements']
        else:
            self._values['measurements'] = value
        
    @measurements.setter
    def measurements(self, value):
        """The measurements property.
        
        Args:
            value (hash). the property value.
        """
        if value == self._defaults['measurements'] and 'measurements' in self._values:
            del self._values['measurements']
        else:
            self._values['measurements'] = value
        
    def _initialize(self):
        """Initializes the current instance of the object.
        """
        pass
    
    def write(self):
        """Writes the contents of this object and returns the content as a dict object.
        
        Returns:
            (dict). the object that represents the same data as the current instance.
        """
        return _write_complex_object(self._defaults, self._values)




# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=no-self-use, line-too-long, protected-access, too-few-public-methods, unused-argument
from knack.log import get_logger

from ..aaz.latest.vm.host.group import Show as _VMHostGroupShow

logger = get_logger(__name__)


class VMHostGroupShow(_VMHostGroupShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)

        args_schema.expand._registered = False

        return args_schema


def convert_show_result_to_snake_case(result):
    new_result = {}
    if 'location' in result:
        new_result['location'] = result['location']

    if 'tags' in result:
        new_result['tags'] = result['tags']

    if 'zones' in result:
        new_result['zones'] = result['zones']

    if 'additionalCapabilities' in result:
        new_result['additional_capabilities'] = result['additionalCapabilities']

        if new_result['additional_capabilities'].get('ultraSSDEnabled'):
            new_result['additional_capabilities']['ultra_ssd_enabled'] = new_result['additional_capabilities']['ultraSSDEnabled']
            new_result['additional_capabilities'].pop('ultraSSDEnabled')

    if 'platformFaultDomainCount' in result:
        new_result['platform_fault_domain_count'] = result['platformFaultDomainCount']

    if 'supportAutomaticPlacement' in result:
        new_result['support_automatic_placement'] = result['supportAutomaticPlacement']
    return new_result



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._recommendation_metadata_operations import RecommendationMetadataOperations
from ._configurations_operations import ConfigurationsOperations
from ._recommendations_operations import RecommendationsOperations
from ._operations import Operations
from ._suppressions_operations import SuppressionsOperations

__all__ = [
    'RecommendationMetadataOperations',
    'ConfigurationsOperations',
    'RecommendationsOperations',
    'Operations',
    'SuppressionsOperations',
]



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from ._access_control_client import KeyVaultAccessControlClient
from ._backup_client import KeyVaultBackupClient
from ._enums import KeyVaultRoleScope, KeyVaultDataAction, KeyVaultSettingType
from ._internal.client_base import ApiVersion
from ._models import (
    KeyVaultBackupResult,
    KeyVaultPermission,
    KeyVaultRoleAssignment,
    KeyVaultRoleAssignmentProperties,
    KeyVaultRoleDefinition,
    KeyVaultSetting,
)
from ._settings_client import KeyVaultSettingsClient


__all__ = [
    "ApiVersion",
    "KeyVaultBackupResult",
    "KeyVaultAccessControlClient",
    "KeyVaultBackupClient",
    "KeyVaultDataAction",
    "KeyVaultPermission",
    "KeyVaultRoleAssignment",
    "KeyVaultRoleAssignmentProperties",
    "KeyVaultRoleDefinition",
    "KeyVaultRoleScope",
    "KeyVaultSetting",
    "KeyVaultSettingsClient",
    "KeyVaultSettingType",
]

from ._version import VERSION
__version__ = VERSION



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, TYPE_CHECKING

from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse
from azure.mgmt.core import ARMPipelineClient
from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy

from . import models as _models
from ._configuration import PostgreSQLManagementClientConfiguration
from ._serialization import Deserializer, Serializer
from .operations import (
    AdministratorsOperations,
    BackupsOperations,
    CheckNameAvailabilityOperations,
    CheckNameAvailabilityWithLocationOperations,
    ConfigurationsOperations,
    DatabasesOperations,
    FirewallRulesOperations,
    FlexibleServerOperations,
    GetPrivateDnsZoneSuffixOperations,
    LocationBasedCapabilitiesOperations,
    LogFilesOperations,
    LtrBackupOperationsOperations,
    MigrationsOperations,
    Operations,
    PostgreSQLManagementClientOperationsMixin,
    PrivateEndpointConnectionOperations,
    PrivateEndpointConnectionsOperations,
    PrivateLinkResourcesOperations,
    QuotaUsagesOperations,
    ReplicasOperations,
    ServerCapabilitiesOperations,
    ServerThreatProtectionSettingsOperations,
    ServersOperations,
    VirtualEndpointsOperations,
    VirtualNetworkSubnetUsageOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials import TokenCredential


class PostgreSQLManagementClient(
    PostgreSQLManagementClientOperationsMixin
):  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """The Microsoft Azure management API provides create, read, update, and delete functionality for
    Azure PostgreSQL resources including servers, databases, firewall rules, VNET rules, security
    alert policies, log files and configurations with new business model.

    :ivar administrators: AdministratorsOperations operations
    :vartype administrators:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.AdministratorsOperations
    :ivar backups: BackupsOperations operations
    :vartype backups: azure.mgmt.rdbms.postgresql_flexibleservers.operations.BackupsOperations
    :ivar location_based_capabilities: LocationBasedCapabilitiesOperations operations
    :vartype location_based_capabilities:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.LocationBasedCapabilitiesOperations
    :ivar server_capabilities: ServerCapabilitiesOperations operations
    :vartype server_capabilities:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.ServerCapabilitiesOperations
    :ivar check_name_availability: CheckNameAvailabilityOperations operations
    :vartype check_name_availability:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.CheckNameAvailabilityOperations
    :ivar check_name_availability_with_location: CheckNameAvailabilityWithLocationOperations
     operations
    :vartype check_name_availability_with_location:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.CheckNameAvailabilityWithLocationOperations
    :ivar configurations: ConfigurationsOperations operations
    :vartype configurations:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.ConfigurationsOperations
    :ivar databases: DatabasesOperations operations
    :vartype databases: azure.mgmt.rdbms.postgresql_flexibleservers.operations.DatabasesOperations
    :ivar firewall_rules: FirewallRulesOperations operations
    :vartype firewall_rules:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.FirewallRulesOperations
    :ivar servers: ServersOperations operations
    :vartype servers: azure.mgmt.rdbms.postgresql_flexibleservers.operations.ServersOperations
    :ivar flexible_server: FlexibleServerOperations operations
    :vartype flexible_server:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.FlexibleServerOperations
    :ivar ltr_backup_operations: LtrBackupOperationsOperations operations
    :vartype ltr_backup_operations:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.LtrBackupOperationsOperations
    :ivar migrations: MigrationsOperations operations
    :vartype migrations:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.MigrationsOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.rdbms.postgresql_flexibleservers.operations.Operations
    :ivar get_private_dns_zone_suffix: GetPrivateDnsZoneSuffixOperations operations
    :vartype get_private_dns_zone_suffix:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.GetPrivateDnsZoneSuffixOperations
    :ivar private_endpoint_connections: PrivateEndpointConnectionsOperations operations
    :vartype private_endpoint_connections:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.PrivateEndpointConnectionsOperations
    :ivar private_endpoint_connection: PrivateEndpointConnectionOperations operations
    :vartype private_endpoint_connection:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.PrivateEndpointConnectionOperations
    :ivar private_link_resources: PrivateLinkResourcesOperations operations
    :vartype private_link_resources:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.PrivateLinkResourcesOperations
    :ivar quota_usages: QuotaUsagesOperations operations
    :vartype quota_usages:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.QuotaUsagesOperations
    :ivar replicas: ReplicasOperations operations
    :vartype replicas: azure.mgmt.rdbms.postgresql_flexibleservers.operations.ReplicasOperations
    :ivar log_files: LogFilesOperations operations
    :vartype log_files: azure.mgmt.rdbms.postgresql_flexibleservers.operations.LogFilesOperations
    :ivar server_threat_protection_settings: ServerThreatProtectionSettingsOperations operations
    :vartype server_threat_protection_settings:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.ServerThreatProtectionSettingsOperations
    :ivar virtual_endpoints: VirtualEndpointsOperations operations
    :vartype virtual_endpoints:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.VirtualEndpointsOperations
    :ivar virtual_network_subnet_usage: VirtualNetworkSubnetUsageOperations operations
    :vartype virtual_network_subnet_usage:
     azure.mgmt.rdbms.postgresql_flexibleservers.operations.VirtualNetworkSubnetUsageOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2023-12-01-preview". Note that overriding
     this default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "TokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = PostgreSQLManagementClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                ARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=base_url, policies=_policies, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.administrators = AdministratorsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.backups = BackupsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.location_based_capabilities = LocationBasedCapabilitiesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.server_capabilities = ServerCapabilitiesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.check_name_availability = CheckNameAvailabilityOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.check_name_availability_with_location = CheckNameAvailabilityWithLocationOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.configurations = ConfigurationsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.databases = DatabasesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.firewall_rules = FirewallRulesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.servers = ServersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.flexible_server = FlexibleServerOperations(self._client, self._config, self._serialize, self._deserialize)
        self.ltr_backup_operations = LtrBackupOperationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.migrations = MigrationsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.get_private_dns_zone_suffix = GetPrivateDnsZoneSuffixOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.private_endpoint_connections = PrivateEndpointConnectionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.private_endpoint_connection = PrivateEndpointConnectionOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.private_link_resources = PrivateLinkResourcesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.quota_usages = QuotaUsagesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.replicas = ReplicasOperations(self._client, self._config, self._serialize, self._deserialize)
        self.log_files = LogFilesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.server_threat_protection_settings = ServerThreatProtectionSettingsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.virtual_endpoints = VirtualEndpointsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.virtual_network_subnet_usage = VirtualNetworkSubnetUsageOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def _send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client._send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> "PostgreSQLManagementClient":
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)



import collections
import copy
from .Utils import _write_complex_object

class Operation(object):
    """Data contract class for type Operation.
    """
    _defaults = collections.OrderedDict([
        ('ai.operation.id', None),
        ('ai.operation.name', None),
        ('ai.operation.parentId', None),
        ('ai.operation.syntheticSource', None),
        ('ai.operation.correlationVector', None)
    ])
    
    def __init__(self):
        """Initializes a new instance of the class.
        """
        self._values = {
        }
        self._initialize()
        
    @property
    def id(self):
        """The id property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'ai.operation.id' in self._values:
            return self._values['ai.operation.id']
        return self._defaults['ai.operation.id']
        
    @id.setter
    def id(self, value):
        """The id property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['ai.operation.id'] and 'ai.operation.id' in self._values:
            del self._values['ai.operation.id']
        else:
            self._values['ai.operation.id'] = value
        
    @property
    def name(self):
        """The name property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'ai.operation.name' in self._values:
            return self._values['ai.operation.name']
        return self._defaults['ai.operation.name']
        
    @name.setter
    def name(self, value):
        """The name property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['ai.operation.name'] and 'ai.operation.name' in self._values:
            del self._values['ai.operation.name']
        else:
            self._values['ai.operation.name'] = value
        
    @property
    def parent_id(self):
        """The parent_id property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'ai.operation.parentId' in self._values:
            return self._values['ai.operation.parentId']
        return self._defaults['ai.operation.parentId']
        
    @parent_id.setter
    def parent_id(self, value):
        """The parent_id property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['ai.operation.parentId'] and 'ai.operation.parentId' in self._values:
            del self._values['ai.operation.parentId']
        else:
            self._values['ai.operation.parentId'] = value
        
    @property
    def synthetic_source(self):
        """The synthetic_source property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'ai.operation.syntheticSource' in self._values:
            return self._values['ai.operation.syntheticSource']
        return self._defaults['ai.operation.syntheticSource']
        
    @synthetic_source.setter
    def synthetic_source(self, value):
        """The synthetic_source property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['ai.operation.syntheticSource'] and 'ai.operation.syntheticSource' in self._values:
            del self._values['ai.operation.syntheticSource']
        else:
            self._values['ai.operation.syntheticSource'] = value
        
    @property
    def correlation_vector(self):
        """The correlation_vector property.
        
        Returns:
            (string). the property value. (defaults to: None)
        """
        if 'ai.operation.correlationVector' in self._values:
            return self._values['ai.operation.correlationVector']
        return self._defaults['ai.operation.correlationVector']
        
    @correlation_vector.setter
    def correlation_vector(self, value):
        """The correlation_vector property.
        
        Args:
            value (string). the property value.
        """
        if value == self._defaults['ai.operation.correlationVector'] and 'ai.operation.correlationVector' in self._values:
            del self._values['ai.operation.correlationVector']
        else:
            self._values['ai.operation.correlationVector'] = value
        
    def _initialize(self):
        """Initializes the current instance of the object.
        """
        pass
    
    def write(self):
        """Writes the contents of this object and returns the content as a dict object.
        
        Returns:
            (dict). the object that represents the same data as the current instance.
        """
        return _write_complex_object(self._defaults, self._values)




# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    AdminCredentials,
    AdminCredentialsForPatch,
    AdministratorMicrosoftEntra,
    AdministratorMicrosoftEntraAdd,
    AdministratorMicrosoftEntraProperties,
    AdministratorMicrosoftEntraPropertiesForAdd,
    AdvancedThreatProtectionSettingsModel,
    AdvancedThreatProtectionSettingsProperties,
    AuthConfig,
    AuthConfigForPatch,
    Backup,
    BackupAutomaticAndOnDemand,
    BackupAutomaticAndOnDemandProperties,
    BackupForPatch,
    BackupRequestBase,
    BackupSettings,
    BackupStoreDetails,
    BackupsLongTermRetentionOperation,
    BackupsLongTermRetentionRequest,
    BackupsLongTermRetentionResponse,
    BackupsLongTermRetentionResponseProperties,
    Capability,
    CapabilityBase,
    CapturedLog,
    CapturedLogProperties,
    CheckNameAvailabilityRequest,
    CheckNameAvailabilityResponse,
    Cluster,
    Configuration,
    ConfigurationForUpdate,
    ConfigurationProperties,
    DataEncryption,
    Database,
    DatabaseMigrationState,
    DatabaseProperties,
    DbLevelValidationStatus,
    DbServerMetadata,
    DelegatedSubnetUsage,
    ErrorAdditionalInfo,
    ErrorDetail,
    ErrorResponse,
    FastProvisioningEditionCapability,
    FirewallRule,
    FirewallRuleProperties,
    HighAvailability,
    HighAvailabilityForPatch,
    ImpactRecord,
    LogSpecification,
    LtrBackupOperationResponseProperties,
    LtrPreBackupRequest,
    LtrPreBackupResponse,
    MaintenanceWindow,
    MaintenanceWindowForPatch,
    MetricSpecification,
    MigrateNetworkStatus,
    Migration,
    MigrationNameAvailability,
    MigrationProperties,
    MigrationPropertiesForPatch,
    MigrationResourceForPatch,
    MigrationSecretParameters,
    MigrationSecretParametersForPatch,
    MigrationStatus,
    MigrationSubstateDetails,
    NameAvailabilityModel,
    NameProperty,
    Network,
    ObjectRecommendation,
    ObjectRecommendationDetails,
    ObjectRecommendationProperties,
    ObjectRecommendationPropertiesAnalyzedWorkload,
    ObjectRecommendationPropertiesImplementationDetails,
    Operation,
    OperationDisplay,
    OperationProperties,
    PrivateEndpoint,
    PrivateEndpointConnection,
    PrivateEndpointConnectionProperties,
    PrivateLinkResource,
    PrivateLinkResourceProperties,
    PrivateLinkServiceConnectionState,
    ProxyResource,
    QuotaUsage,
    Replica,
    Resource,
    RestartParameter,
    Server,
    ServerEditionCapability,
    ServerForPatch,
    ServerProperties,
    ServerPropertiesForPatch,
    ServerSku,
    ServerSkuCapability,
    ServerVersionCapability,
    ServiceSpecification,
    Sku,
    SkuForPatch,
    Storage,
    StorageEditionCapability,
    StorageMbCapability,
    StorageTierCapability,
    SupportedFeature,
    SystemData,
    TrackedResource,
    TuningOptions,
    TuningOptionsProperties,
    UserAssignedIdentity,
    UserIdentity,
    ValidationDetails,
    ValidationMessage,
    ValidationSummaryItem,
    VirtualEndpoint,
    VirtualEndpointResourceForPatch,
    VirtualEndpointResourceProperties,
    VirtualNetworkSubnetUsageModel,
    VirtualNetworkSubnetUsageParameter,
)

from ._enums import (  # type: ignore
    AzureManagedDiskPerformanceTier,
    BackupType,
    Cancel,
    CapabilityStatus,
    CheckNameAvailabilityReason,
    ConfigurationDataType,
    CreateMode,
    CreateModeForPatch,
    CreatedByType,
    DataEncryptionType,
    EncryptionKeyStatus,
    ExecutionStatus,
    FailoverMode,
    FastProvisioningSupport,
    FeatureStatus,
    GeographicallyRedundantBackup,
    GeographicallyRedundantBackupSupport,
    HighAvailabilityMode,
    HighAvailabilityState,
    IdentityType,
    LocationRestricted,
    LogicalReplicationOnSourceServer,
    MicrosoftEntraAuth,
    MigrateRolesAndPermissions,
    MigrationDatabaseState,
    MigrationListFilter,
    MigrationMode,
    MigrationNameAvailabilityReason,
    MigrationOption,
    MigrationState,
    MigrationSubstate,
    NetworkMigrationState,
    OnlineStorageResizeSupport,
    OperationOrigin,
    OverwriteDatabasesOnTargetServer,
    PasswordBasedAuth,
    PostgreSqlFlexibleServerHighAvailabilityMode,
    PostgresMajorVersion,
    PrincipalType,
    PrivateEndpointConnectionProvisioningState,
    PrivateEndpointServiceConnectionStatus,
    ReadReplicaPromoteMode,
    ReadReplicaPromoteOption,
    RecommendationTypeEnum,
    RecommendationTypeParameterEnum,
    ReplicationRole,
    ReplicationState,
    ServerPublicNetworkAccessState,
    ServerState,
    SkuTier,
    SourceType,
    SslMode,
    StartDataMigration,
    StorageAutoGrow,
    StorageAutoGrowthSupport,
    StorageType,
    ThreatProtectionName,
    ThreatProtectionState,
    TriggerCutover,
    TuningOptionParameterEnum,
    ValidationState,
    VirtualEndpointType,
    ZoneRedundantHighAvailabilityAndGeographicallyRedundantBackupSupport,
    ZoneRedundantHighAvailabilitySupport,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AdminCredentials",
    "AdminCredentialsForPatch",
    "AdministratorMicrosoftEntra",
    "AdministratorMicrosoftEntraAdd",
    "AdministratorMicrosoftEntraProperties",
    "AdministratorMicrosoftEntraPropertiesForAdd",
    "AdvancedThreatProtectionSettingsModel",
    "AdvancedThreatProtectionSettingsProperties",
    "AuthConfig",
    "AuthConfigForPatch",
    "Backup",
    "BackupAutomaticAndOnDemand",
    "BackupAutomaticAndOnDemandProperties",
    "BackupForPatch",
    "BackupRequestBase",
    "BackupSettings",
    "BackupStoreDetails",
    "BackupsLongTermRetentionOperation",
    "BackupsLongTermRetentionRequest",
    "BackupsLongTermRetentionResponse",
    "BackupsLongTermRetentionResponseProperties",
    "Capability",
    "CapabilityBase",
    "CapturedLog",
    "CapturedLogProperties",
    "CheckNameAvailabilityRequest",
    "CheckNameAvailabilityResponse",
    "Cluster",
    "Configuration",
    "ConfigurationForUpdate",
    "ConfigurationProperties",
    "DataEncryption",
    "Database",
    "DatabaseMigrationState",
    "DatabaseProperties",
    "DbLevelValidationStatus",
    "DbServerMetadata",
    "DelegatedSubnetUsage",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "FastProvisioningEditionCapability",
    "FirewallRule",
    "FirewallRuleProperties",
    "HighAvailability",
    "HighAvailabilityForPatch",
    "ImpactRecord",
    "LogSpecification",
    "LtrBackupOperationResponseProperties",
    "LtrPreBackupRequest",
    "LtrPreBackupResponse",
    "MaintenanceWindow",
    "MaintenanceWindowForPatch",
    "MetricSpecification",
    "MigrateNetworkStatus",
    "Migration",
    "MigrationNameAvailability",
    "MigrationProperties",
    "MigrationPropertiesForPatch",
    "MigrationResourceForPatch",
    "MigrationSecretParameters",
    "MigrationSecretParametersForPatch",
    "MigrationStatus",
    "MigrationSubstateDetails",
    "NameAvailabilityModel",
    "NameProperty",
    "Network",
    "ObjectRecommendation",
    "ObjectRecommendationDetails",
    "ObjectRecommendationProperties",
    "ObjectRecommendationPropertiesAnalyzedWorkload",
    "ObjectRecommendationPropertiesImplementationDetails",
    "Operation",
    "OperationDisplay",
    "OperationProperties",
    "PrivateEndpoint",
    "PrivateEndpointConnection",
    "PrivateEndpointConnectionProperties",
    "PrivateLinkResource",
    "PrivateLinkResourceProperties",
    "PrivateLinkServiceConnectionState",
    "ProxyResource",
    "QuotaUsage",
    "Replica",
    "Resource",
    "RestartParameter",
    "Server",
    "ServerEditionCapability",
    "ServerForPatch",
    "ServerProperties",
    "ServerPropertiesForPatch",
    "ServerSku",
    "ServerSkuCapability",
    "ServerVersionCapability",
    "ServiceSpecification",
    "Sku",
    "SkuForPatch",
    "Storage",
    "StorageEditionCapability",
    "StorageMbCapability",
    "StorageTierCapability",
    "SupportedFeature",
    "SystemData",
    "TrackedResource",
    "TuningOptions",
    "TuningOptionsProperties",
    "UserAssignedIdentity",
    "UserIdentity",
    "ValidationDetails",
    "ValidationMessage",
    "ValidationSummaryItem",
    "VirtualEndpoint",
    "VirtualEndpointResourceForPatch",
    "VirtualEndpointResourceProperties",
    "VirtualNetworkSubnetUsageModel",
    "VirtualNetworkSubnetUsageParameter",
    "AzureManagedDiskPerformanceTier",
    "BackupType",
    "Cancel",
    "CapabilityStatus",
    "CheckNameAvailabilityReason",
    "ConfigurationDataType",
    "CreateMode",
    "CreateModeForPatch",
    "CreatedByType",
    "DataEncryptionType",
    "EncryptionKeyStatus",
    "ExecutionStatus",
    "FailoverMode",
    "FastProvisioningSupport",
    "FeatureStatus",
    "GeographicallyRedundantBackup",
    "GeographicallyRedundantBackupSupport",
    "HighAvailabilityMode",
    "HighAvailabilityState",
    "IdentityType",
    "LocationRestricted",
    "LogicalReplicationOnSourceServer",
    "MicrosoftEntraAuth",
    "MigrateRolesAndPermissions",
    "MigrationDatabaseState",
    "MigrationListFilter",
    "MigrationMode",
    "MigrationNameAvailabilityReason",
    "MigrationOption",
    "MigrationState",
    "MigrationSubstate",
    "NetworkMigrationState",
    "OnlineStorageResizeSupport",
    "OperationOrigin",
    "OverwriteDatabasesOnTargetServer",
    "PasswordBasedAuth",
    "PostgreSqlFlexibleServerHighAvailabilityMode",
    "PostgresMajorVersion",
    "PrincipalType",
    "PrivateEndpointConnectionProvisioningState",
    "PrivateEndpointServiceConnectionStatus",
    "ReadReplicaPromoteMode",
    "ReadReplicaPromoteOption",
    "RecommendationTypeEnum",
    "RecommendationTypeParameterEnum",
    "ReplicationRole",
    "ReplicationState",
    "ServerPublicNetworkAccessState",
    "ServerState",
    "SkuTier",
    "SourceType",
    "SslMode",
    "StartDataMigration",
    "StorageAutoGrow",
    "StorageAutoGrowthSupport",
    "StorageType",
    "ThreatProtectionName",
    "ThreatProtectionState",
    "TriggerCutover",
    "TuningOptionParameterEnum",
    "ValidationState",
    "VirtualEndpointType",
    "ZoneRedundantHighAvailabilityAndGeographicallyRedundantBackupSupport",
    "ZoneRedundantHighAvailabilitySupport",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class Create(AAZCommand):
    """Create a new SSH public key resource.

    :example: Create a new SSH public key resource.
        az sshkey create --resource-group myResourceGroup --ssh-public-key-name mySshPublicKeyName --location westus --public-key {ssh-rsa public key}
    """

    _aaz_info = {
        "version": "2025-04-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.compute/sshpublickeys/{}", "2025-04-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.ssh_public_key_name = AAZStrArg(
            options=["-n", "--name", "--ssh-public-key-name"],
            help="The name of the SSH public key.",
            required=True,
        )

        # define Arg Group "Parameters"

        _args_schema = cls._args_schema
        _args_schema.location = AAZResourceLocationArg(
            arg_group="Parameters",
            help="The geo-location where the resource lives",
            required=True,
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            arg_group="Parameters",
            help="Resource tags.",
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg()

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.public_key = AAZStrArg(
            options=["--public-key"],
            arg_group="Properties",
            help="SSH public key used to authenticate to a virtual machine through ssh. If this property is not initially provided when the resource is created, the publicKey property will be populated when generateKeyPair is called. If the public key is provided upon resource creation, the provided public key needs to be at least 2048-bit and in ssh-rsa format.",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.SshPublicKeysCreate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class SshPublicKeysCreate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200, 201]:
                return self.on_200_201(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/sshPublicKeys/{sshPublicKeyName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "sshPublicKeyName", self.ctx.args.ssh_public_key_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-04-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("location", AAZStrType, ".location", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("publicKey", AAZStrType, ".public_key")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200_201.tags = AAZDictType()
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200_201.properties
            properties.public_key = AAZStrType(
                serialized_name="publicKey",
            )

            system_data = cls._schema_on_200_201.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            tags = cls._schema_on_200_201.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""


__all__ = ["Create"]



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import Any, Callable, Dict, List, Optional, TypeVar

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from .._serialization import Serializer
from .._vendor import ApiManagementClientMixinABC, _convert_request, _format_url_section

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_list_by_service_request(
    resource_group_name: str, service_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2022-08-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ApiManagement/service/{serviceName}/networkstatus",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "serviceName": _SERIALIZER.url(
            "service_name",
            service_name,
            "str",
            max_length=50,
            min_length=1,
            pattern=r"^[a-zA-Z](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?$",
        ),
    }

    _url: str = _format_url_section(_url, **path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_list_by_location_request(
    resource_group_name: str, service_name: str, location_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2022-08-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ApiManagement/service/{serviceName}/locations/{locationName}/networkstatus",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "serviceName": _SERIALIZER.url(
            "service_name",
            service_name,
            "str",
            max_length=50,
            min_length=1,
            pattern=r"^[a-zA-Z](?:[a-zA-Z0-9-]*[a-zA-Z0-9])?$",
        ),
        "locationName": _SERIALIZER.url("location_name", location_name, "str", min_length=1),
    }

    _url: str = _format_url_section(_url, **path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


class NetworkStatusOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.apimanagement.ApiManagementClient`'s
        :attr:`network_status` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_by_service(
        self, resource_group_name: str, service_name: str, **kwargs: Any
    ) -> List[_models.NetworkStatusContractByLocation]:
        """Gets the Connectivity Status to the external resources on which the Api Management service
        depends from inside the Cloud Service. This also returns the DNS Servers as visible to the
        CloudService.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param service_name: The name of the API Management service. Required.
        :type service_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of NetworkStatusContractByLocation or the result of cls(response)
        :rtype: list[~azure.mgmt.apimanagement.models.NetworkStatusContractByLocation]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[List[_models.NetworkStatusContractByLocation]] = kwargs.pop("cls", None)

        request = build_list_by_service_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.list_by_service.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("[NetworkStatusContractByLocation]", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_by_service.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ApiManagement/service/{serviceName}/networkstatus"
    }

    @distributed_trace
    def list_by_location(
        self, resource_group_name: str, service_name: str, location_name: str, **kwargs: Any
    ) -> _models.NetworkStatusContract:
        """Gets the Connectivity Status to the external resources on which the Api Management service
        depends from inside the Cloud Service. This also returns the DNS Servers as visible to the
        CloudService.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param service_name: The name of the API Management service. Required.
        :type service_name: str
        :param location_name: Location in which the API Management service is deployed. This is one of
         the Azure Regions like West US, East US, South Central US. Required.
        :type location_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: NetworkStatusContract or the result of cls(response)
        :rtype: ~azure.mgmt.apimanagement.models.NetworkStatusContract
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.NetworkStatusContract] = kwargs.pop("cls", None)

        request = build_list_by_location_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            location_name=location_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.list_by_location.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("NetworkStatusContract", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_by_location.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ApiManagement/service/{serviceName}/locations/{locationName}/networkstatus"
    }



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=unused-argument, line-too-long
from azure.cli.core.util import CLIError, user_confirmation
from azure.mgmt import postgresqlflexibleservers as postgresql_flexibleservers
from knack.log import get_logger
from .identity_commands import flexible_server_identity_update
from .parameter_commands import _update_parameters
from .._client_factory import cf_postgres_flexible_servers
from ..utils.validators import validate_resource_group, validate_citus_cluster

logger = get_logger(__name__)


def flexible_server_fabric_mirroring_start(cmd, client, resource_group_name, server_name, database_names, yes=False):
    validate_resource_group(resource_group_name)
    validate_citus_cluster(cmd, resource_group_name, server_name)
    flexible_servers_client = cf_postgres_flexible_servers(cmd.cli_ctx, '_')
    server = flexible_servers_client.get(resource_group_name, server_name)

    if server.high_availability.mode != "Disabled" and server.version not in ["17", "18"]:
        # disable fabric mirroring on HA server
        raise CLIError("Fabric mirroring is not supported on servers with high availability enabled.")

    databases = ','.join(database_names)
    user_confirmation("Are you sure you want to prepare and enable your server" +
                      " '{0}' in resource group '{1}' for mirroring of databases '{2}'.".format(server_name, resource_group_name, databases) +
                      " This requires restart.", yes=yes)

    if (server.identity is None or 'SystemAssigned' not in server.identity.type):
        logger.warning('Enabling system assigned managed identity on the server.')
        flexible_server_identity_update(cmd, flexible_servers_client, resource_group_name, server_name, 'Enabled')

    logger.warning('Updating necessary server parameters.')
    source = "user-override"
    configuration_name = "azure.fabric_mirror_enabled"
    value = "on"
    _update_parameters(cmd, client, server_name, configuration_name, resource_group_name, source, value)
    configuration_name = "azure.mirror_databases"
    value = databases
    return _update_parameters(cmd, client, server_name, configuration_name, resource_group_name, source, value)


def flexible_server_fabric_mirroring_stop(cmd, client, resource_group_name, server_name, yes=False):
    validate_resource_group(resource_group_name)
    validate_citus_cluster(cmd, resource_group_name, server_name)

    flexible_servers_client = cf_postgres_flexible_servers(cmd.cli_ctx, '_')
    server = flexible_servers_client.get(resource_group_name, server_name)

    if server.high_availability.mode != "Disabled" and server.version not in ["17", "18"]:
        # disable fabric mirroring on HA server
        raise CLIError("Fabric mirroring is not supported on servers with high availability enabled.")

    user_confirmation("Are you sure you want to disable mirroring for server '{0}' in resource group '{1}'".format(server_name, resource_group_name), yes=yes)

    configuration_name = "azure.fabric_mirror_enabled"
    parameters = postgresql_flexibleservers.models.Configuration(
        value="off",
        source="user-override"
    )

    return client.begin_update(resource_group_name, server_name, configuration_name, parameters)


def flexible_server_fabric_mirroring_update_databases(cmd, client, resource_group_name, server_name, database_names, yes=False):
    validate_resource_group(resource_group_name)
    validate_citus_cluster(cmd, resource_group_name, server_name)

    flexible_servers_client = cf_postgres_flexible_servers(cmd.cli_ctx, '_')
    server = flexible_servers_client.get(resource_group_name, server_name)

    if server.high_availability.mode != "Disabled" and server.version not in ["17", "18"]:
        # disable fabric mirroring on HA server
        raise CLIError("Fabric mirroring is not supported on servers with high availability enabled.")

    databases = ','.join(database_names)
    user_confirmation("Are you sure for server '{0}' in resource group '{1}' you want to update the databases being mirrored to be '{2}'"
                      .format(server_name, resource_group_name, databases), yes=yes)

    configuration_name = "azure.mirror_databases"
    parameters = postgresql_flexibleservers.models.Configuration(
        value=databases,
        source="user-override"
    )

    return client.begin_update(resource_group_name, server_name, configuration_name, parameters)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from typing import Any, Callable, Dict, Optional, TypeVar

from azure.core import AsyncPipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ..._utils.serialization import Deserializer, Serializer
from ...operations._access_review_history_definition_instance_operations import build_generate_download_uri_request
from .._configuration import AuthorizationManagementClientConfiguration

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class AccessReviewHistoryDefinitionInstanceOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.authorization.aio.AuthorizationManagementClient`'s
        :attr:`access_review_history_definition_instance` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: AuthorizationManagementClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def generate_download_uri(
        self, history_definition_id: str, instance_id: str, **kwargs: Any
    ) -> _models.AccessReviewHistoryInstance:
        """Generates a uri which can be used to retrieve review history data. This URI has a TTL of 1 day
        and can be retrieved by fetching the accessReviewHistoryDefinition object.

        :param history_definition_id: The id of the access review history definition. Required.
        :type history_definition_id: str
        :param instance_id: The id of the access review history definition instance to generate a URI
         for. Required.
        :type instance_id: str
        :return: AccessReviewHistoryInstance or the result of cls(response)
        :rtype: ~azure.mgmt.authorization.models.AccessReviewHistoryInstance
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2021-12-01-preview"))
        cls: ClsType[_models.AccessReviewHistoryInstance] = kwargs.pop("cls", None)

        _request = build_generate_download_uri_request(
            history_definition_id=history_definition_id,
            instance_id=instance_id,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorDefinition, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("AccessReviewHistoryInstance", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, TYPE_CHECKING

from msrest import Deserializer, Serializer

from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.mgmt.core import AsyncARMPipelineClient

from .. import models
from ._configuration import EventGridManagementClientConfiguration
from .operations import ChannelsOperations, DomainEventSubscriptionsOperations, DomainTopicEventSubscriptionsOperations, DomainTopicsOperations, DomainsOperations, EventChannelsOperations, EventSubscriptionsOperations, ExtensionTopicsOperations, Operations, PartnerConfigurationsOperations, PartnerDestinationsOperations, PartnerNamespacesOperations, PartnerRegistrationsOperations, PartnerTopicEventSubscriptionsOperations, PartnerTopicsOperations, PrivateEndpointConnectionsOperations, PrivateLinkResourcesOperations, SystemTopicEventSubscriptionsOperations, SystemTopicsOperations, TopicEventSubscriptionsOperations, TopicTypesOperations, TopicsOperations, VerifiedPartnersOperations

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential

class EventGridManagementClient:    # pylint: disable=too-many-instance-attributes
    """Azure EventGrid Management Client.

    :ivar channels: ChannelsOperations operations
    :vartype channels: azure.mgmt.eventgrid.aio.operations.ChannelsOperations
    :ivar domains: DomainsOperations operations
    :vartype domains: azure.mgmt.eventgrid.aio.operations.DomainsOperations
    :ivar domain_topics: DomainTopicsOperations operations
    :vartype domain_topics: azure.mgmt.eventgrid.aio.operations.DomainTopicsOperations
    :ivar event_channels: EventChannelsOperations operations
    :vartype event_channels: azure.mgmt.eventgrid.aio.operations.EventChannelsOperations
    :ivar event_subscriptions: EventSubscriptionsOperations operations
    :vartype event_subscriptions: azure.mgmt.eventgrid.aio.operations.EventSubscriptionsOperations
    :ivar domain_topic_event_subscriptions: DomainTopicEventSubscriptionsOperations operations
    :vartype domain_topic_event_subscriptions:
     azure.mgmt.eventgrid.aio.operations.DomainTopicEventSubscriptionsOperations
    :ivar topic_event_subscriptions: TopicEventSubscriptionsOperations operations
    :vartype topic_event_subscriptions:
     azure.mgmt.eventgrid.aio.operations.TopicEventSubscriptionsOperations
    :ivar domain_event_subscriptions: DomainEventSubscriptionsOperations operations
    :vartype domain_event_subscriptions:
     azure.mgmt.eventgrid.aio.operations.DomainEventSubscriptionsOperations
    :ivar system_topic_event_subscriptions: SystemTopicEventSubscriptionsOperations operations
    :vartype system_topic_event_subscriptions:
     azure.mgmt.eventgrid.aio.operations.SystemTopicEventSubscriptionsOperations
    :ivar partner_topic_event_subscriptions: PartnerTopicEventSubscriptionsOperations operations
    :vartype partner_topic_event_subscriptions:
     azure.mgmt.eventgrid.aio.operations.PartnerTopicEventSubscriptionsOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.eventgrid.aio.operations.Operations
    :ivar partner_configurations: PartnerConfigurationsOperations operations
    :vartype partner_configurations:
     azure.mgmt.eventgrid.aio.operations.PartnerConfigurationsOperations
    :ivar partner_destinations: PartnerDestinationsOperations operations
    :vartype partner_destinations:
     azure.mgmt.eventgrid.aio.operations.PartnerDestinationsOperations
    :ivar partner_namespaces: PartnerNamespacesOperations operations
    :vartype partner_namespaces: azure.mgmt.eventgrid.aio.operations.PartnerNamespacesOperations
    :ivar partner_registrations: PartnerRegistrationsOperations operations
    :vartype partner_registrations:
     azure.mgmt.eventgrid.aio.operations.PartnerRegistrationsOperations
    :ivar partner_topics: PartnerTopicsOperations operations
    :vartype partner_topics: azure.mgmt.eventgrid.aio.operations.PartnerTopicsOperations
    :ivar private_endpoint_connections: PrivateEndpointConnectionsOperations operations
    :vartype private_endpoint_connections:
     azure.mgmt.eventgrid.aio.operations.PrivateEndpointConnectionsOperations
    :ivar private_link_resources: PrivateLinkResourcesOperations operations
    :vartype private_link_resources:
     azure.mgmt.eventgrid.aio.operations.PrivateLinkResourcesOperations
    :ivar system_topics: SystemTopicsOperations operations
    :vartype system_topics: azure.mgmt.eventgrid.aio.operations.SystemTopicsOperations
    :ivar topics: TopicsOperations operations
    :vartype topics: azure.mgmt.eventgrid.aio.operations.TopicsOperations
    :ivar extension_topics: ExtensionTopicsOperations operations
    :vartype extension_topics: azure.mgmt.eventgrid.aio.operations.ExtensionTopicsOperations
    :ivar topic_types: TopicTypesOperations operations
    :vartype topic_types: azure.mgmt.eventgrid.aio.operations.TopicTypesOperations
    :ivar verified_partners: VerifiedPartnersOperations operations
    :vartype verified_partners: azure.mgmt.eventgrid.aio.operations.VerifiedPartnersOperations
    :param credential: Credential needed for the client to connect to Azure.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: Subscription credentials that uniquely identify a Microsoft Azure
     subscription. The subscription ID forms part of the URI for every service call.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2021-10-15-preview". Note that overriding
     this default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = EventGridManagementClientConfiguration(credential=credential, subscription_id=subscription_id, **kwargs)
        self._client = AsyncARMPipelineClient(base_url=base_url, config=self._config, **kwargs)

        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.channels = ChannelsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.domains = DomainsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.domain_topics = DomainTopicsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.event_channels = EventChannelsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.event_subscriptions = EventSubscriptionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.domain_topic_event_subscriptions = DomainTopicEventSubscriptionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.topic_event_subscriptions = TopicEventSubscriptionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.domain_event_subscriptions = DomainEventSubscriptionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.system_topic_event_subscriptions = SystemTopicEventSubscriptionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.partner_topic_event_subscriptions = PartnerTopicEventSubscriptionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.partner_configurations = PartnerConfigurationsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.partner_destinations = PartnerDestinationsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.partner_namespaces = PartnerNamespacesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.partner_registrations = PartnerRegistrationsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.partner_topics = PartnerTopicsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.private_endpoint_connections = PrivateEndpointConnectionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.private_link_resources = PrivateLinkResourcesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.system_topics = SystemTopicsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.topics = TopicsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.extension_topics = ExtensionTopicsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.topic_types = TopicTypesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.verified_partners = VerifiedPartnersOperations(self._client, self._config, self._serialize, self._deserialize)


    def _send_request(
        self,
        request: HttpRequest,
        **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client._send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/python/protocol/quickstart

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, **kwargs)

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> "EventGridManagementClient":
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details) -> None:
        await self._client.__aexit__(*exc_details)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access

from azure.cli.command_modules.monitor.aaz.latest.monitor.diagnostic_settings._create \
    import Create as _DiagnosticSettingsCreate
from azure.cli.command_modules.monitor.operations.diagnostics_settings import (
    create_resource_parameters, update_resource_parameters,
)
from azure.cli.core.azclierror import ArgumentUsageError
from knack.util import CLIError


class DiagnosticSettingsCreate(_DiagnosticSettingsCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        arg_schema = super()._build_arguments_schema(*args, **kwargs)
        create_resource_parameters(arg_schema, arg_group="Target Resource")

        from azure.cli.core.aaz import AAZBoolArg
        arg_schema.export_to_resource_specific = AAZBoolArg(
            options=['--export-to-resource-specific'],
            help="Indicate that the export to LA must be done to a resource specific table, a.k.a. "
                 "dedicated or fixed schema table, as opposed to the default dynamic schema table called "
                 "AzureDiagnostics. This argument is effective only when the argument --workspace is also given. "
                 "Allowed values: false, true."
        )
        arg_schema.log_analytics_destination_type._registered = False
        arg_schema.service_bus_rule_id._registered = False
        return arg_schema

    def pre_operations(self):
        ctx = self.ctx
        from azure.cli.core.aaz import has_value
        from azure.mgmt.core.tools import is_valid_resource_id, resource_id, parse_resource_id
        update_resource_parameters(ctx)
        args = ctx.args
        rg = args.resource_group_name.to_serialized_data()

        if not has_value(rg):
            rg = parse_resource_id(args.resource.to_serialized_data())['resource_group']
            args.resource_group_name = rg

        storage_account = args.storage_account.to_serialized_data()
        if has_value(storage_account) and not is_valid_resource_id(storage_account):
            storage_account = resource_id(subscription=ctx.subscription_id,
                                          resource_group=rg,
                                          namespace='microsoft.Storage',
                                          type='storageAccounts',
                                          name=storage_account)
            args.storage_account = storage_account

        workspace = args.workspace.to_serialized_data()
        if has_value(workspace) and not is_valid_resource_id(workspace):
            workspace = resource_id(subscription=ctx.subscription_id,
                                    resource_group=rg,
                                    namespace='microsoft.OperationalInsights',
                                    type='workspaces',
                                    name=workspace)
            args.workspace = workspace

        event_hub = args.event_hub.to_serialized_data()
        if has_value(event_hub) and is_valid_resource_id(event_hub):
            event_hub = parse_resource_id(event_hub)['name']
            args.event_hub = event_hub

        event_hub_rule = args.event_hub_rule.to_serialized_data()
        if has_value(event_hub_rule):
            if not is_valid_resource_id(event_hub_rule):
                if not has_value(event_hub):
                    raise CLIError('usage error: --event-hub-rule ID | --event-hub-rule NAME --event-hub NAME')
                # use value from --event-hub if the rule is a name
                event_hub_rule = resource_id(
                    subscription=ctx.subscription_id,
                    resource_group=rg,
                    namespace='Microsoft.EventHub',
                    type='namespaces',
                    name=event_hub,
                    child_type_1='AuthorizationRules',
                    child_name_1=event_hub_rule)
                args.event_hub_rule = event_hub_rule

            elif not has_value(event_hub):
                # extract the event hub name from `--event-hub-rule` if provided as an ID
                event_hub = parse_resource_id(event_hub_rule)['name']
                args.event_hub = event_hub
        if not (has_value(storage_account) or has_value(workspace) or has_value(event_hub)):
            raise CLIError(
                'usage error - expected one or more:  --storage-account NAME_OR_ID | --workspace NAME_OR_ID '
                '| --event-hub NAME_OR_ID | --event-hub-rule ID')

        export_to_resource_specific = args.export_to_resource_specific.to_serialized_data()
        if has_value(export_to_resource_specific) and export_to_resource_specific:
            args.log_analytics_destination_type = 'Dedicated'
            if not has_value(workspace):
                raise ArgumentUsageError('usage error: --workspace and --export-to-specific-resource')
        else:
            args.log_analytics_destination_type = None
